from struct import Struct


class Mapping(object):

    RED = 0
    GREEN = 1
    BLUE = 2
    CHANNELS = 3

    def __init__(self):
        """create a new color mapping for an EtherPix distributor"""
        self._base = 0.0
        self._gamma = 1.0
        self._factor = 1.0
        self._precompute()

    def set_params(self, base, gamma, factor):
        """set mapping parameters"""
        self._base = base
        self._gamma = gamma
        self._factor = factor
        self._precompute()

    def _precompute(self):
        """pre-compute mapping table"""
        # display_value := base + factor * input_value ^ (1 / gamma)
        gamma_1 = 1.0 / self._gamma
        s = Struct("!B")
        tab = []
        for v in range(256):
            display_val = self._base + self._factor * (v / 255.0) ** gamma_1
            if display_val < 0:
                display_no = 0
            elif display_val > 1.0:
                display_no = 255
            else:
                display_no = int(round(display_val * 255.0))
            tab.append(s.pack(display_no))
        self.table = tuple(tab)

